﻿// ------------------------------------------------------------------------------------
//
// FishFace.CS                                                   Ulrich Müller 17.11.09
//                                                               Aspire C# 2005 Express
// Basisklasse FishFace - umFish50.DLL -ftMscLib.DLL
//
// ------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

namespace FishFaceTX {
  public class FishFace {
    #region ---- externe Deklarationen -------
    // --- umFish50.DLL Deklarationen -------------------
    [DllImport("ftMscLib.dll")]
    private static extern uint ftxGetLibVersionStr(StringBuilder sb, uint len);
    [DllImport("umFish50.DLL")]
    private static extern int txOpenController(int ComNr);
    [DllImport("umFish50.DLL")]
    private static extern int txCloseController();

    [DllImport("umFish50.DLL")]
    private static extern int txGetAnalog(int ctrId, int inpNr);
    [DllImport("umFish50.DLL")]
    private static extern int txGetDistance(int ctrId, int inpNr);
    [DllImport("umFish50.DLL")]
    private static extern int txGetInput(int ctrId, int inpNr);
    [DllImport("umFish50.DLL")]
    private static extern int txGetTrack(int ctrId, int inpNr);
    [DllImport("umFish50.DLL")]
    private static extern int txGetVoltage(int ctrId, int inpNr);

    [DllImport("umFish50.DLL")]
    private static extern int txGetCounter(int ctrId, int cntNr);
    [DllImport("umFish50.DLL")]
    private static extern int txClearCounter(int ctrId, int cntNr);

    [DllImport("umFish50.DLL")]
    private static extern int txSetLamp(int ctrId, int outNr, int OnOff);
    [DllImport("umFish50.DLL")]
    private static extern int txSetMotor(int ctrId, int motNr, int dir, int speed);
    [DllImport("umFish50.DLL")]
    private static extern int txStartMotor(int ctrId, int motNr,
      int dir, int speed, int iCount, int robMode);

    [DllImport("umFish50.DLL")]
    private static extern int txIsMotorReady(int ctrId, int motNr);


    [DllImport("kernel32")]
    internal static extern void Sleep(uint mSek);
    [DllImport("kernel32")]
    internal static extern uint GetTickCount();
    [DllImport("user32")]
    internal static extern uint GetAsyncKeyState(uint vKey);
    internal const uint ESCAPE = 0x1B;

    public bool IsESC {
      get {
        return (GetAsyncKeyState(ESCAPE) != 0);
      }
    }
    #endregion
    private const uint txOK           = 0x00000000;
    private const uint txError        = 0xE0000001;
    private const uint txPollInterval = 12;
    
    private bool FOpen     = false;
    private bool FNotHalt  = false;
    private bool FDoEvents = true;

    public FishFace() {
      FOpen = false;
      FNotHalt = false;
      FDoEvents = true;
    }
    public bool NotHalt {
      get {
        return FNotHalt;
      }
      set {
        FNotHalt = value;
      }
    }
    public static string Version {
      get {
        return "5.0.2.1";
      }
    }
    public static string LibVersion {
      get {
        StringBuilder sb = new StringBuilder(null, 26);
        return ((ftxGetLibVersionStr(sb, 25) != 0) ? sb.ToString() : null);
      }
    }

    /// <summary>
    /// Herstellen einer Verbindung (USB / Bluetooth) zum TX Controller
    /// </summary>
    /// <param name="ComName">Anschlußname</param>
    public void OpenController(string ComName) {
      int ComNr = Convert.ToInt32(ComName.Substring(3));
      if ((uint)txOpenController(ComNr) == txError)
        throw (new FishFaceException("ControllerProblem.Open"));
      FNotHalt  = false;
      FOpen     = true;
      FDoEvents = true;
    }
    /// <summary>
    /// Beenden einer bestehenden Verbindung zum TX Controller
    /// </summary>
    public void CloseController() {
      if (FOpen) {
        if ((uint)txCloseController() == txError)
          throw (new FishFaceException("ControllerProblem.Close"));
        FNotHalt = false;
        FOpen = false;
      }
    }

    public bool Finish() {
      return Finish(Unv.I0);
    }
    public bool Finish(Unv InputNr) {
      return Finish(Ctr.Main, InputNr);
    }
    /// <summary>
    /// Feststellen, ob ein Abbruchwunsch für das Programm vorliegt
    /// NotHalt = true, Esc-Taste, Abbruch-Taster
    /// </summary>
    /// <param name="ctrId">Controller ID</param>
    /// <param name="InputNr">I-Eingang D5K</param>
    /// <returns>true, wenn Abbruchwunsch vorliegt</returns>
    public bool Finish(Ctr ctrId, Unv InputNr) {
      // --- Feststellen, ob ein Abbruchwunsch für das Programm vorliegt :
      //     NotHalt = true (schließt ESC-Taste ein), optional auch Taster
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new FishFaceException("KeinOpen.Finish"));
      if (FNotHalt || IsESC) return true;
      try {
        if (((int)InputNr >= 0) && GetInput(ctrId, InputNr)) return true;
      }
      catch (FishFaceException) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.Finish"));
      }
      return false;
    }
    /// <summary>
    /// Anhalten des Programmablaufs
    /// Unterbrechbar, abbrechbar
    /// </summary>
    /// <param name="mSek">Zeitdauer in msec</param>
    public void Pause(int mSek) {
      uint sZeit = GetTickCount();
      do {
        if (FDoEvents) Application.DoEvents();
        if (FNotHalt || IsESC) return;
        Sleep(txPollInterval);
      } while (GetTickCount() - sZeit < mSek);
    }
    
    public int GetAnalog(Unv InputNr) {
      return GetAnalog(Ctr.Main, InputNr);
    }
    public int GetAnalog(Ctr ctrId, Unv InputNr) {
      // --- Auslesen eines analogen Widerstandswertes von einem Universaleingang
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new FishFaceException("KeinOpen.GetAnalog"));
      int res = txGetAnalog((int)ctrId, (int)InputNr);
      if ((uint)res == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.GetAnalog"));
      }
      return res;
    }
    public int GetDistance(Unv InputNr) {
      return GetDistance(Ctr.Main, InputNr);
    }
    public int GetDistance(Ctr ctrId, Unv InputNr) {
      // --- Auslesen des aktuellen Wertes [cm] des angeschlossenen Ultraschallsensors
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new FishFaceException("KeinOpen.GetDistance"));
      int res = txGetDistance((int)ctrId, (int)InputNr);
      if ((uint)res == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.GetDistance"));
      }
      return res;
    }
    public bool GetInput(Unv InputNr) {
      return GetInput(Ctr.Main, InputNr);
    }
    public bool GetInput(Ctr ctrId, Unv InputNr) {
      // --- Einlesen eines binären Widerstandswertes von einem Universaleingang
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new FishFaceException("KeinOpen.GetInput"));
      int res = txGetInput((int)ctrId, (int)InputNr);
      if ((uint)res == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.GetInput"));
      }
      return res == 0 ? false : true;
    }
    public bool GetTrack(Unv InputNr) {
      return GetTrack(Ctr.Main, InputNr);
    }
    public bool GetTrack(Ctr ctrId, Unv InputNr) {
      // --- Einlesen eines Spurwertes von einem Universaleingang ---
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new FishFaceException("KeinOpen.GetTrack"));
      int res = txGetTrack((int)ctrId, (int)InputNr);
      if ((uint)res == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.GetTrack"));
      }
      return res > 0 ? true : false;
    }
    public int GetVoltage(Unv InputNr) {
      return GetVoltage(Ctr.Main, InputNr);
    }
    public int GetVoltage(Ctr ctrId, Unv InputNr) {
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new FishFaceException("KeinOpen.GetVoltage"));
      int res = txGetVoltage((int)ctrId, (int)InputNr);
      if ((uint)res == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.GetVoltage"));
      }
      return res;
    }
    
    public int GetCounter(Cnt Counter) {
      return GetCounter(Ctr.Main, Counter);
    }
    public int GetCounter(Ctr ctrId, Cnt Counter) {
      if (!FOpen) throw (new FishFaceException("KeinOpen.GetCounter"));
      int res = txGetCounter((int)ctrId, (int)Counter);
      if ((uint)res == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.GetCounter"));
      }
      return res;
    }
    public void ClearCounter(Cnt Counter) {
      ClearCounter(Ctr.Main, Counter);
    }
    public void ClearCounter(Ctr ctrId, Cnt Counter) {
      if (!FOpen) throw (new FishFaceException("KeinOpen.ClearCounter"));
      if ((uint)txClearCounter((int)ctrId, (int)Counter) == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.ClearCounter"));
      }
    }

    public void SetLamp(Out LampNr, Dir OnOff) {
      SetLamp(Ctr.Main, LampNr, OnOff, OnOff == Dir.On ? 512 : 0);
    }
    public void SetLamp(Ctr ctrId, Out LampNr, Dir OnOff) {
      SetLamp(ctrId, LampNr, OnOff, OnOff == Dir.On ? 512 : 0);
    }
    public void SetLamp(Out LampNr, int Power) {
      SetLamp(Ctr.Main, LampNr, Power > 0 ? Dir.On : Dir.Off, Power);
    }
    public void SetLamp(Ctr ctrId, Out LampNr, int Power) {
      SetLamp(ctrId, LampNr, Power > 0 ? Dir.On : Dir.Off, Power);
    }
    public void SetLamp(Ctr ctrId, Out LampNr, Dir OnOff, int Power) {
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new FishFaceException("KeinOpen.SetLamp"));
      if ((uint)txSetLamp((int)ctrId, (int)LampNr, OnOff == Dir.On ? Power : 0) == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.SetLamp"));
      }
    }

    public void SetMotor(Mot MotNr, Dir Direction) {
      SetMotor(Ctr.Main, MotNr, Direction, Direction == Dir.Off ? 0 : 512);
    }
    public void SetMotor(Ctr ctrId, Mot MotNr, Dir Direction) {
      SetMotor(ctrId, MotNr, Direction, Direction == Dir.Off ? 0 : 512);
    }
    public void SetMotor(Mot MotNr, Dir Direction, int Speed) {
      SetMotor(Ctr.Main, MotNr, Direction, Speed);
    }
    public void SetMotor(Ctr ctrId, Mot MotNr, Dir Direction, int Speed) {
      StartMotor(ctrId, MotNr, Direction, Speed, 0);
    }
    public void StartMotor(Mot MotNr, Dir Direction, int Speed, int ICount) {
      StartenMotor(Ctr.Main, MotNr, Direction, Speed, ICount, 0);
    }
    public void StartMotor(Ctr ctrId, Mot MotNr, Dir Direction, int Speed, int ICount) {
      StartenMotor(ctrId, MotNr, Direction, Speed, ICount, 0);
    }
    public void StartRobMotor(Mot MotNr, Dir Direction, int Speed, int ICount) {
      StartenMotor(Ctr.Main, MotNr, Direction, Speed, ICount, 1);
    }
    public void StartRobMotor(Ctr ctrId, Mot MotNr, Dir Direction,
      int Speed, int ICount) {
      StartenMotor(ctrId, MotNr, Direction, Speed, ICount, 1);
    }
    private void StartenMotor(Ctr ctrId, Mot MotNr, Dir Direction,
      int Speed, int ICount, int RobMode) {
      // --- Starten eines M-Ausganges, ggf. mit zugeordetem Impulszähler
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new FishFaceException("KeinOpen.StartMotor"));
      if ((uint)txStartMotor((int)ctrId, (int)MotNr,
      (int)Direction, Speed, ICount, RobMode) == txError) {
        FOpen = false;
        throw (new FishFaceException("ControllerProblem.StartMotor"));
      }
    }

    public int WaitForMotor(Mot MotNr) {
      return WaitForMotor(Ctr.Main, MotNr);
    }
    public int WaitForMotor(Ctr ctrId, Mot MotNr) {
      if (!FOpen) throw (new FishFaceException("KeinOpen.WaitForMotor"));
      int res = 0;
      do {
        res = txIsMotorReady((int)ctrId, (int)MotNr);
        if ((uint)res == txError) {
          FOpen = false;
          throw (new FishFaceException("ControllerProblem.WaitForMotor"));
        }
        if (FDoEvents) Application.DoEvents();
      } while (res < 0 && !FNotHalt && !IsESC);
      return res;
    }
    public void WaitForMotors(params Mot[] MotorNrs) {
      WaitForMotors(Ctr.Main, MotorNrs);
    }
    public void WaitForMotors(Ctr ctrId, params Mot[] MotorNrs) {
      if (!FOpen) throw (new FishFaceException("KeinOpen.WaitForMotors"));
      int c = 0;
      int res;
      do {
        c = 0;
        for (int i = 0; i < MotorNrs.Length; i++) {
          res = txIsMotorReady((int)ctrId, (int)MotorNrs[i]);
          if ((uint)res == txError) {
            FOpen = false;
            throw (new FishFaceException("ControllerProblem.WaitForMotors"));
          }
          c += res < 0 ? 1 : 0;
        }
        if (c == 0) return;
        if (FDoEvents) Application.DoEvents();
      } while (!FNotHalt && !IsESC);
    }
    public void WaitForCount(Cnt CounterNr, int NrOfChanges) {
      WaitForCount(Ctr.Main, CounterNr, NrOfChanges, Unv.I0);
    }
    public void WaitForCount(Cnt CounterNr, int NrOfChanges, Unv TermInputNr) {
      WaitForCount(Ctr.Main, CounterNr, NrOfChanges, TermInputNr);
    }
    public void WaitForCount(Ctr ctrId, Cnt CounterNr, int NrOfChanges) {
      WaitForCount(ctrId, CounterNr, NrOfChanges, Unv.I0);
    }
    public void WaitForCount(Ctr ctrId, Cnt CounterNr, int NrOfChanges, Unv TermInputNr) {
      int count = 0;
      if (!FOpen) throw (new FishFaceException("KeinOpen.WaitForCount"));
      ClearCounter(ctrId, CounterNr);
      do {
        GetCounter(ctrId, CounterNr);
        Sleep(txPollInterval);
        if (FNotHalt) return;
        if (TermInputNr > 0) if (GetInput(ctrId, TermInputNr)) return;
        if (FDoEvents) Application.DoEvents();
      } while (count < NrOfChanges);
    }
    public void WaitForInput(Unv InputNr) {
      WaitForInput(Ctr.Main, InputNr, true);
    }
    public void WaitForInput(Ctr ctrId, Unv InputNr) {
      WaitForInput(ctrId, InputNr, true);
    }
    public void WaitForInput(Unv InputNr, bool OnOff) {
      WaitForInput(Ctr.Main, InputNr, OnOff);
    }
    public void WaitForInput(Ctr ctrId, Unv InputNr, bool OnOff) {
      if (!FOpen) throw (new FishFaceException("KeinOpen.WaitForInput"));
      try {
        do {
          if (FNotHalt || IsESC) return;
          Sleep(txPollInterval);
          if (FDoEvents) Application.DoEvents();
        } while (!GetInput(ctrId, InputNr) == OnOff);
      }
      catch (FishFaceException) {
        throw (new FishFaceException("InterfaceProblem.WaitForInput"));
      }
    }
    public void WaitForHigh(Unv InputNr) {
      WaitForHigh(Ctr.Main, InputNr);
    }
    public void WaitForHigh(Ctr ctrId, Unv InputNr) {
      WaitForInput(ctrId, InputNr, false);
      WaitForInput(ctrId, InputNr, true);
    }
    public void WaitForLow(Unv InputNr) {
      WaitForLow(Ctr.Main, InputNr);
    }
    public void WaitForLow(Ctr ctrId, Unv InputNr) {
      WaitForInput(ctrId, InputNr, true);
      WaitForInput(ctrId, InputNr, false);
    }
  }
}
