﻿// --------------------------------------------------------------------------------------
//
// LightBarrier.CS                                                 Ulrich Müller 01.11.09
//                                                                 Aspire C# 2005 Express
// Lichtschranke aus Photowiderstand und Lampe, wenn die Lampe nicht angegeben wird (O0),
//               ist sie selber zu versorgen (Anschluß direkt an Akku)
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  public class LightBarrier : DeviceBase {

    public delegate void Changed(LightBarrier sender);
    public event Changed ChangedToFree;
    public event Changed ChangedToBroken;

    private Lamp            lightSource;
    private PhotoTransistor photoSensor;

    public LightBarrier(FishControl fc, Ctr ctrId, Out LightSource, Unv PhotoSensor)
      : this(fc, false, ctrId, LightSource, PhotoSensor) {
    }
    public LightBarrier(FishControl fc, bool WithEvents,
      Ctr ctrId, Out LightSource, Unv PhotoSensor)
      : base(fc, false, (int)ctrId, 103) {
      base.deviceType  = "LightBarrier";
      this.lightSource = LightSource == Out.O0 ? null : new Lamp(fc, ctrId, LightSource);
      this.photoSensor = new PhotoTransistor(fc, ctrId, PhotoSensor);
      base.withEvents  = WithEvents;
      if (WithEvents) {
        this.photoSensor.WithEvents = true;
        photoSensor.ChangedToFalse += new BinaryInput.Changed(BrokenEvent);
        photoSensor.ChangedToTrue += new BinaryInput.Changed(FreeEvent);
      }
    }

    protected override void EventLoop() {
    }
    private void FreeEvent(BinaryInput sender) {
      if (ChangedToFree != null) ChangedToFree(this);
    }
    private void BrokenEvent(BinaryInput sender) {
      if (ChangedToBroken != null) ChangedToBroken(this);
    }

    public override string ConnectorName {
      get {
        return base.deviceType;
      }
    }
    public int PhotoNumber {
      get {
        return photoSensor.ConnectorNumber;
      }
    }
    public string PhotoName {
      get {
        return photoSensor.ConnectorName;
      }
    }
    public int LightNumber {
      get {
        return lightSource == null ? 0 : lightSource.ConnectorNumber;
      }
    }
    public string LightName {
      get {
        return lightSource == null ? "none" : lightSource.ConnectorName;
      }
    }
    public override int ThreadInterval {
      get {
        return base.ThreadInterval;
      }
      set {
        base.ThreadInterval = value;
        photoSensor.ThreadInterval = value;
      }
    }
    public bool IsBroken {
      get {
        return !photoSensor.IsTrue;
      }
    }
    public bool IsFree {
      get {
        return photoSensor.IsTrue;
      }
    }

    public void On() {
      if (lightSource != null) {
        lightSource.On();
        tx.Pause(1000);
      }
    }
    public void Off() {
      if (lightSource != null) lightSource.Off();
    }
    public void WaitForBroken() {
      photoSensor.WaitForFalse();
    }
    public void WaitForFree() {
      photoSensor.WaitForTrue();
    }
    public void WaitForPassed() {
      photoSensor.WaitForHigh();
    }
  }
}
