﻿// --------------------------------------------------------------------------------------
//
// MonoOutputs.CS                                                  Ulrich Müller 01.11.09
//                                                                 Aspire C# 2005 Express
// Devices, die von MonoOutput abgeleitet sind (auch Combined)
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  public class Lamp : MonoOutput {

    private int timeOn, timeOff;
    private bool blinkingOn;
    private Thread BlinkingThread;

    public Lamp(FishControl fc, Ctr ctrId, Out OutName) 
      : base(fc, ctrId, OutName) {
    }

    public void BlinkingOn(int Time) {
      this.BlinkingOn(Time, Time);
    }
    public void BlinkingOn(int TimeOn, int TimeOff) {
      this.timeOn = TimeOn > 11 ? TimeOn : 11;
      this.timeOff = TimeOff > 11 ? TimeOff : 11;
      BlinkingThread = new Thread(new ThreadStart(BlinkingLoop));
      BlinkingThread.IsBackground = true;
      blinkingOn = true;
      BlinkingThread.Start();
    }
    public void BlinkingOff() {
      blinkingOn = false;
    }
    private void BlinkingLoop() {
      bool FlipFlop = true;
      int BlinkTime = timeOn;
      while (!tx.NotHalt && !tx.IsESC && blinkingOn) {
        if (FlipFlop) {
          this.On();
          BlinkTime = timeOn;
        }
        else {
          this.Off();
          BlinkTime = timeOff;
        }
        FlipFlop = !FlipFlop;
        Thread.Sleep(BlinkTime);
      }
    }
  }

  public class Lights : DeviceBase {

    private Lamp[] LampList;
    private int anzLamp;

    public Lights(FishControl fc, Ctr ctrId, params Out[] Lamps)
      : base(fc, false, (int)ctrId, 104) {
      base.deviceType = "Lights";
      anzLamp = Lamps.Length;
      LampList = new Lamp[anzLamp];
      for (int i = 0; i < anzLamp; i++) 
        LampList[i] = new Lamp(fc, ctrId, Lamps[i]);
    }

    protected override void EventLoop() {
      throw new Exception("The method or operation is not implemented.");
    }

    public override string ConnectorName {
      get {
        return base.deviceType;
      }
    }

    public void Switch(params Dir[] SwitchList) {
      for (int i = 0; i < SwitchList.Length; i++) LampList[i].Switch(SwitchList[i]);
    }
    public void Switch(int mSek, params Dir[] SwitchList) {
      for (int i = 0; i < SwitchList.Length; i++) LampList[i].Switch(SwitchList[i]);
      tx.Pause(mSek);
      for (int i = 0; i < SwitchList.Length; i++) LampList[i].Switch(Dir.Off);
    }
    public void SwitchOff() {
      for (int i = 0; i < anzLamp; i++) LampList[i].Off();
    }
    public Lamp this[int index] {
      get {
        return LampList[index];
      }
    }
  }


  public class Magnet : MonoOutput {
    public Magnet(FishControl fc, Ctr ctrId, Out OutName) 
      : base(fc, ctrId, OutName) {
    }
  }

  public class PneuValve : MonoOutput {
    public PneuValve(FishControl fc, Ctr ctrId, Out OutName) 
      : base(fc, ctrId, OutName) {
    }
  }

  public class Buzzer : MonoOutput {
    public Buzzer(FishControl fc, Ctr ctrId, Out OutName)
      : base(fc, ctrId, OutName) {
    }

    public new void On(int Duration) {
      this.On(Duration, 512);
    }
    public void On(int Duration, int Power) {
      base.On(Power);
      tx.Pause(Duration);
      base.Off();
    }

  }
}
