﻿// --------------------------------------------------------------------------------------
//
// TrailSensor.CS                                                  Ulrich Müller 01.03.09
//                                                                 Aspire C# 2005 Express
// Device TrailSensor : Spurverfolgung mit Belegung von zwei I-Eingängen
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  public class TrailSensor : DeviceBase {

    public delegate void Changed(TrailSensor sender);
    public event Changed ChangedToLeft;
    public event Changed ChangedToRight;
    public event Changed ChangedToTrail;
    public event Changed ChangedToOff;
    public delegate void ChangedTrail(TrailSensor sender, TrailState trailState);
    public event         ChangedTrail ChangedState;

    private HalfTrack trackLeft;
    private HalfTrack trackRight;

    private TrailState oldTrailState, actTrailState;


    public TrailSensor(FishControl fc, Ctr ctrId, Unv TrackLeft, Unv TrackRight)
      : this(fc, false, ctrId, TrackLeft, TrackRight) {
    }
    public TrailSensor(FishControl fc, bool WithEvents, 
      Ctr ctrId, Unv TrackLeft, Unv TrackRight) 
      : base(fc, WithEvents, (int)ctrId, 108) {
      base.withEvents = WithEvents;
      base.deviceType = "TrailSensor";
      trackLeft  = new HalfTrack(fc, ctrId, TrackLeft);
      trackRight = new HalfTrack(fc, ctrId, TrackRight);
      base.threadInterval = 22;
    }

    protected override void EventLoop() {
      oldTrailState = TrailState.Undef;
      while (!tx.NotHalt && !tx.IsESC) {
        actTrailState = TrailPosition;
        if (actTrailState != oldTrailState) {
          if (ChangedState != null) ChangedState(this, actTrailState);
          switch (actTrailState) {
            case TrailState.Left:
              if (ChangedToLeft != null) ChangedToLeft(this);
              break;
            case TrailState.Right:
              if (ChangedToRight != null) ChangedToRight(this);
              break;
            case TrailState.OnTrail:
              if (ChangedToTrail != null) ChangedToTrail(this);
              break;
            case TrailState.OffTrail:
              if (ChangedToOff != null) ChangedToOff(this);
              break;
            default:
              break;
          }
        }
        oldTrailState = actTrailState;
      }
      Thread.Sleep(base.threadInterval);
    }

    public string ConNameLeft {
      get {
        return trackLeft.ConnectorName;
      }
    }
    public string ConNameRight {
      get {
        return trackRight.ConnectorName;
      }
    }
    public override string ConnectorName {
      get {
        return base.deviceType;
      }
    }
    public int ConNumberLeft {
      get {
        return trackLeft.ConnectorNumber;
      }
    }
    public int ConNumberRight {
      get {
        return trackRight.ConnectorNumber;
      }
    }

    public bool IsLeft {
      get {
        return trackLeft.IsTrue && trackRight.IsFalse ? false : true;
      }
    }
    public bool IsRight {
      get {
        return trackLeft.IsFalse && trackRight.IsTrue ? false : true;
      }
    }
    public bool IsOnTrail {
      get {
        return trackLeft.IsFalse && trackRight.IsFalse ? false : true;
      }
    }
    public bool IsOffTrail {
      get {
        return trackLeft.IsTrue && trackRight.IsTrue ? false : true;
      }
    }
    public TrailState TrailPosition {
      get {
        if (IsLeft) return TrailState.Left;
        else if (IsRight) return TrailState.Right;
        else if (IsOnTrail) return TrailState.OnTrail;
        else return TrailState.OffTrail;
      }
    }
  }
}
